@@ -4,10 +4,10 @@
 library(tensorflow)
 library(Rcpp)
 
-Sys.setenv("PKG_CXXFLAGS"="-std=c++11")
+Sys.setenv("PKG_CXXFLAGS"="-std=c++17")
 sourceCpp("src/gdInt.cpp")
 
-generate <- function(outGenerativeDataFileName, numberOfIterations, inKeepProbability, columnIndices) {
+generate <- function(generativeDataFileName, columnIndices, generateParameters) {
   gdTf <- tf$compat$v1
   gdTf$disable_v2_behavior()
   
@@ -16,12 +16,12 @@ generate <- function(outGenerativeDataFileName, numberOfIterations, inKeepProbab
   cInitEpochs <- -100
 
   batchSize <- gdGetBatchSize();
-  dataDimension <- gdGetDataSourceDimension()
+  dimension <- gdGetDataSourceDimension()
   
-  if(dataDimension > 1 && length(columnIndices) != 2) {
+  if(dimension > 1 && length(columnIndices) != 2) {
     message("size of vector columnIndices must be equal to two\n")
     return()
-  } else if (dataDimension == 1 && length(columnIndices) != 1) {
+  } else if (dimension == 1 && length(columnIndices) != 1) {
     message("size of vector columnIndices must be equal to one\n")
     return()
   }
@@ -29,9 +29,10 @@ generate <- function(outGenerativeDataFileName, numberOfIterations, inKeepProbab
   columnNames <- gdGetNumberVectorIndexNames(columnIndices)
   
   cOutputLayerSize <- gdGetDataSourceDimension()
-  cLr <- 0.00005
-  cLrg <- 0.00005
 
+  cLr <- generateParameters$learningRate
+  cLrg <- generateParameters$learningRate
+  
   gdTf$reset_default_graph()
 
   x <- gdTf$placeholder(gdTf$float32, shape = c(batchSize, cOutputLayerSize))
@@ -88,7 +89,7 @@ generate <- function(outGenerativeDataFileName, numberOfIterations, inKeepProbab
   dTrainer <- gdTf$train$RMSPropOptimizer(cLr)$minimize(dLoss, var_list = dVars)
   gTrainer <- gdTf$train$RMSPropOptimizer(cLrg)$minimize(-dLoss, var_list = gVars)
 
-  epochs <- numberOfIterations
+  epochs <- generateParameters$numberOfIterations
   init <- gdTf$global_variables_initializer()
   session <- gdTf$Session()
   session$run(init)
@@ -97,26 +98,24 @@ generate <- function(outGenerativeDataFileName, numberOfIterations, inKeepProbab
 
   epoch <- 1
   for(epoch in cInitEpochs:epochs) {
-
     samples <- NULL
-    i <- 1
     for(i in 1:cNumberOfBatchesPerEpoch) {
-      samples <- gdDataSourceGetNormalizedDataRandom(batchSize)
-      samples <- array_reshape(samples, c(batchSize, dataDimension))
-      noise <- array_reshape(runif(batchSize * dataDimension, -1.0, 1.0), c(batchSize, dataDimension))
-
+      samples <- array_reshape(gdDataSourceGetNormalizedDataRandom(batchSize), c(batchSize, dimension))
+      noise <- array_reshape(runif(batchSize * dimension, -1.0, 1.0), c(batchSize, dimension))
+      
       if(epoch < 1) {
-        samples <- array_reshape(runif(batchSize * dataDimension, 0.0, 1.0), c(batchSize, dataDimension))
+        samples <- array_reshape(runif(batchSize * dimension, 0.0, 1.0), c(batchSize, dimension))
       }
-
-      session$run(dTrainer, feed_dict = dict(x = samples, noiseInput = noise, keepProbability = inKeepProbability))
-      session$run(gTrainer, feed_dict= dict(noiseInput = noise, keepProbability = inKeepProbability))
+      
+      session$run(dTrainer, feed_dict = dict(x = samples, noiseInput = noise, keepProbability = generateParameters$keepProbability))
+      session$run(gTrainer, feed_dict= dict(noiseInput = noise, keepProbability = generateParameters$keepProbability))
     }
     message(epoch)
-
-    noise <- array_reshape(runif(batchSize * dataDimension, -1.0, 1.0), c(batchSize, dataDimension))
-
-    r <- session$run(list(D_pred, gGenerator), feed_dict = dict(x = samples, noiseInput = noise, keepProbability = inKeepProbability))
+    
+    noise <- array_reshape(runif(batchSize * dimension, -1.0, 1.0), c(batchSize, dimension))
+    
+    #r <- session$run(list(D_pred, gGenerator), feed_dict = dict(x = samples, noiseInput = noise, keepProbability = inKeepProbability))
+    r <- session$run(list(D_pred, gGenerator), feed_dict = dict(noiseInput = noise, keepProbability = generateParameters$keepProbability))
     gs <- r[[2]]
     gp <- array(1, c(batchSize))
     gp <- array_reshape(r[[1]], c(batchSize))
@@ -126,7 +125,7 @@ generate <- function(outGenerativeDataFileName, numberOfIterations, inKeepProbab
         k <- k + 1
       }
     }
-    gsp <- array(0, c(k, dataDimension))
+    gsp <- array(0, c(k, dimension))
     l <- 1
     for(j in 1:batchSize) {
       if(gp[j] >= 0.5) {
@@ -134,18 +133,19 @@ generate <- function(outGenerativeDataFileName, numberOfIterations, inKeepProbab
         l <- l + 1
       }
     }
-    gsp <- array_reshape(gsp, c(k * dataDimension))
+    gsp <- array_reshape(gsp, c(k * dimension))
     
     if(epoch >= 1) {
       gdAddValueRows(gsp)
     }
-    if(dataDimension == 1) {
+    
+    if(dimension == 1) {
       plot(gs[, columnIndices[1]], array(0, batchSize), main = "gdGenerate", cex.main = 1.0, font.main = 1, xlim = c(0.0, 1.0), ylim = c(0.0, 1.0), col = ifelse(gp >= 0.5, "green", "red"), xlab = columnNames[1], ylab = columnNames[2])
     } else {
       plot(gs[, columnIndices[1]], gs[, columnIndices[2]], main = "gdGenerate", cex.main = 1.0, font.main = 1, xlim = c(0.0, 1.0), ylim = c(0.0, 1.0), col = ifelse(gp >= 0.5, "green", "red"), xlab = columnNames[1], ylab = columnNames[2])      
     }
     legend("topleft", legend = c("gd positive", "gd negative", "ds"), col = c("green", "red", "blue"), pch = c(1, 1, 1), bty = "n", horiz = TRUE)
-    if(dataDimension == 1) {
+    if(dimension == 1) {
       points(samples[, columnIndices[1]], array(0, batchSize), col = "blue")
     } else {
       points(samples[, columnIndices[1]], samples[, columnIndices[2]], col = "blue")
@@ -153,12 +153,31 @@ generate <- function(outGenerativeDataFileName, numberOfIterations, inKeepProbab
     
     if(epoch >= 1) {
       if(epoch %% cWriteEpochsModulo == 0) {
-        gdGenerativeDataWrite(outGenerativeDataFileName)
+        gdGenerativeDataWrite(generativeDataFileName)
       }
     }
   }
 
-  gdGenerativeDataWrite(outGenerativeDataFileName)
+  gdGenerativeDataWrite(generativeDataFileName)
+}
+
+#' Specify parameters for generation of generative data
+#' 
+#' Specify parameters for training of neural networks used for generation of generative data.
+#' These parameters are passed to function gdGenerate().
+#'
+#' @param numberOfIterations Number of training steps
+#' @param learningRate Learning rate for training of neural networks
+#' @param keepProbability Value in the range of 0 to 1 which is used to train generalized neural networks.
+#' 
+#' @return List of parameters for generation of generative data
+#' @export
+#' 
+#' @examples
+#' \dontrun{
+#' gdPlotParameters(50, c(0.75), c("red", "green"))}
+gdGenerateParameters <- function(numberOfIterations = 25000, learningRate = 0.0001, keepProbability = 0.95) {
+  parameters <- list(numberOfIterations = numberOfIterations, learningRate = learningRate, keepProbability = keepProbability )
 }
 
 #' Generate generative data for a data source
@@ -167,27 +186,39 @@ generate <- function(outGenerativeDataFileName, numberOfIterations, inKeepProbab
 #' write generated data to a file in binary format. When a higher number of iterations is used 
 #' the distribution of generated data gets closer to that of the data source.
 #'
-#' @param dataSourceFileName Name of data source file
 #' @param generativeDataFileName Name of generative data file
-#' @param numberOfIterations Number of iterations.
-#' In this version the limit of number of iterations is set to 50000.
-#' @param keepProbability Value in the range of 0 to 1 which is used in training of neural networks to train generalized networks.
+#' @param dataSourceFileName Name of data source file
 #' @param columnIndices Vector of two column indices that are used to plot two-dimensional projections of normalized generated generative data and
 #' data source for a training step. Indices refer to indices of active columns of data source. 
-#'
+#' @param generateParameters Generation of generative data parameters, see function gdGenerateParameters().
+#' 
 #' @return None
 #' @export
 #'
 #' @examples
-#' \dontrun{gdGenerate("iris4d.bin", "gd.bin", 2500, 0.95, c(1, 2))}
-gdGenerate <- function(dataSourceFileName, generativeDataFileName, numberOfIterations, keepProbability, columnIndices) {
-  if(numberOfIterations > gdGetMaxSize() / gdGetBatchSize()) {
+#' \dontrun{
+#' generateParameters = gdGenerateParameters(numberOfIterations = 5000)
+#' gdGenerate("gd.bin", "ds.bin", c(1, 2), generateParameters)}
+gdGenerate <- function(generativeDataFileName,
+                       dataSourceFileName,
+                       columnIndices,
+                       generateParameters = gdGenerateParameters(numberOfIterations = 25000, learningRate = 0.0001, keepProbability = 0.95)) {
+  if(generateParameters$numberOfIterations > gdGetMaxSize() / gdGetBatchSize()) {
     message("Max number of iterations exceeded")
     return()
   }
-  gdReset()
   
+  start <- Sys.time()
+  
+  gdReset()
   gdDataSourceRead(dataSourceFileName)
   gdCreateGenerativeData()
-  generate(generativeDataFileName, numberOfIterations, keepProbability, columnIndices)
+  generate(generativeDataFileName, columnIndices, generateParameters)
+  
+  end <- Sys.time()
+  message(round(difftime(end, start, units = "secs"), 3), " seconds")
 }
+
+#devtools::build_manual()
+
+#library(ganGenerativeData)
